import {
  Component,
  EventEmitter,
  Output,
  Input,
  SimpleChanges,
} from '@angular/core';
import {
  ReactiveFormsModule,
  FormBuilder,
  FormGroup,
  Validators,
} from '@angular/forms';
import { CommonModule } from '@angular/common';
import { Units } from '../../../shared/enums/units.enum';
import {
  decimalNumberValidator,
  conditionalRequiredValidator,
} from '../../../core/utils/validators';
import { TableData } from '../../../core/interfaces/table-data';
import { RawMaterialsService } from '../../../core/services/api/rawmaterials.service';
import { InventoryService } from '../../../core/services/enums/inventory.service';
import { ProvidersService } from '../../../core/services/api/providers.service';
import { DeleteModalComponent } from '../../../shared/components/delete-modal/delete-modal.component';
import { formatDate, formatDateString } from '../../../core/utils/mapping.utils';

@Component({
  selector: 'app-raw-material',
  standalone: true,
  imports: [ReactiveFormsModule, CommonModule, DeleteModalComponent],
  templateUrl: './raw-material.component.html',
  styleUrl: './raw-material.component.scss',
})
export class RawMaterialComponent {
  @Input() productInput: TableData = {};
  @Output() close = new EventEmitter<void>();
  productForm: FormGroup;
  addTypeForm: FormGroup;

  inventoryTypes: string[] = [];
  inputsTypes: string[] = [];
  units: string[] = [];
  addTypeModal: boolean = false;

  providers: TableData[] = [];

  avaibleFreezed: boolean = false;
  isToggled: boolean = false;
  confirmDeleteModal: boolean = false;

  constructor(
    private RawMaterialsServices: RawMaterialsService,
    private formBuilder: FormBuilder,
    private InventoryService: InventoryService,
    private ProvidersService: ProvidersService
  ) {
    this.productForm = this.formBuilder.group(
      {
        name: [{ value: '', disabled: !this.isToggled }, [Validators.required]],
        type: [
          { value: '', disabled: !this.isToggled },
          [Validators.required, Validators.pattern('^[a-zA-Z ]*$')],
        ],
        quantity: [
          { value: '', disabled: !this.isToggled },
          [Validators.required],
        ],
        unit: [{ value: '', disabled: !this.isToggled }, [Validators.required]],
        cost: [
          { value: '', disabled: !this.isToggled },
          [Validators.required, decimalNumberValidator],
        ],
        description: [
          { value: '', disabled: !this.isToggled },
          [Validators.pattern(/^[a-zA-Z0-9\s]+$/)],
        ],
        freezed: [{ value: false, disabled: true }, [Validators.required]],
        expirationDate: [{ value: '', disabled: true }],
        provider: [
          { value: '', disabled: !this.isToggled },
          [Validators.required],
        ],
      },
      {
        validators: conditionalRequiredValidator('expirationDate', 'freezed'),
      }
    );
    this.addTypeForm = this.formBuilder.group({
      newType: ['', [Validators.required, Validators.pattern('^[a-zA-Z ]*$')]],
    });
  }

  ngOnInit() {
    //console.log('productInput', this.productInput);
    this.InventoryService.chargeRawMaterialTypes();
    this.getProviders();
    this.loadProductTypes();
    this.units = Object.values(Units);
    this.productForm.get('type')?.valueChanges.subscribe(() => {
      this.updateFreezedState();
    });
    this.productForm.get('freezed')?.valueChanges.subscribe((isFreezed) => {
      this.updateExpirationDateState(isFreezed);
    });
  }

  ngOnChanges(changes: SimpleChanges) {
    if (changes['productInput'] && changes['productInput'].currentValue) {
      //console.log(this.transformProductInput(this.productInput));
      this.productForm.patchValue(this.transformProductInput(this.productInput));
    }
  }

  toggleEditMode() {
    this.isToggled = !this.isToggled;
    if (this.isToggled) {
      this.productForm.enable(); // Habilita todos los controles del formulario
    } else {
      this.productForm.disable(); // Deshabilita todos los controles del formulario
    }
    this.updateFreezedState();
  }

  closeModal() {
    this.close.emit();
  }

  confirmDelete() {
    this.confirmDeleteModal = true;
  }

  closeDeleteModal() {
    this.confirmDeleteModal = false;
  }

  deleteProduct() {
    //Call backend
    this.RawMaterialsServices.deleteRawMaterial(
      this.productInput['id']
    ).subscribe(
      () => {
        this.closeModal();
      },
      (error) => {
        console.error('Error al eliminar la materia prima', error);
      }
    );
  }

  openAddTypeModal() {
    this.addTypeModal = true;
  }

  closeAddTypeModal() {
    this.addTypeModal = false;
  }


  addType() {
    if (this.addTypeForm.valid) {
      const newType = this.addTypeForm.value.newType;
      this.InventoryService.addRawMaterialType(newType);
      this.closeAddTypeModal();
    } else {
      console.error('Formulario de tipo inválido.');
    }
  }

  onSubmit() {
    if (this.productForm.valid) {
      const providerId = this.productForm.get('provider')?.value;
      const selectedProvider = this.providers.find((provider) => {
        return Number(provider['id']) === Number(providerId);
      });
      const rawMaterial: TableData = {
        id: this.productInput['id'],
        name: this.productForm.get('name')?.value,
        type: this.productForm.get('type')?.value,
        quantity: this.productForm.get('quantity')?.value,
        unit: this.productForm.get('unit')?.value,
        cost: this.productForm.get('cost')?.value,
        descripction: this.productForm.get('description')?.value || '',
        freeze: this.productForm.get('freezed')?.value,
        expirationDate: this.productForm.get('expirationDate')?.value,
        provider: selectedProvider || {},
        admissionDate: formatDateString(this.productInput['admissionDate']),
        lastModificationDate: formatDateString(this.productInput['lastModificationDate'])
      }
      this.RawMaterialsServices.updateRawMaterial(rawMaterial).subscribe(
        () => {
          this.closeModal();
        },
        (error) => {
          console.error('Error al actualizar la materia prima', error);}
      );
    }
  }

  private transformProductInput(data: TableData): any {
    return {
      name: data['name'] ?? '',
      type: data['type'] ?? '',
      quantity: data['quantity'] ? String(data['quantity']) : '',
      unit: data['unit'] ?? '',
      cost: data['cost'] ? String(data['cost']) : '',
      freezed: data['freeze'] ?? false,
      expirationDate: formatDate(data['expirationDate']),
      provider: data['provider'].id ?? '',
    };
  }

  private getProviders() {
    this.ProvidersService.getProviders().subscribe((res: any[]) => {
      this.providers = res.sort((a, b) => a.name.localeCompare(b.name));
    });
  }

  private loadProductTypes(): void {
    this.InventoryService.getInputsTypes().subscribe((types) => {
      this.inventoryTypes = types.sort((a, b) => a.localeCompare(b));
    });
  }

  private updateExpirationDateState(isFreezed: boolean): void {
    const expirationDateControl = this.productForm.get('expirationDate');
    if (isFreezed) {
      expirationDateControl?.enable();
    } else {
      expirationDateControl?.disable();
    }
  }
  private updateFreezedState(): void {
    const typeControl = this.productForm.get('type');
    const freezedControl = this.productForm.get('freezed');

    if (typeControl && freezedControl) {
      const typeValue = typeControl.value;
      const isTypeInInputs = this.inputsTypes.includes(typeValue);

      // Deshabilitar el campo 'freezed' si el tipo está en inputsTypes, habilitar si no está en inputsTypes
      if (isTypeInInputs) {
        freezedControl.disable();
        // También podemos establecer un valor predeterminado para 'freezed' cuando está deshabilitado
        if (freezedControl.value) {
          freezedControl.setValue(false);
        }
      } else {
        freezedControl.enable();
      }
    }
  }
}
